@@ -1,4 +1,4 @@
-# Copyright (c) 2022, Riverbank Computing Limited
+# Copyright (c) 2023, Riverbank Computing Limited
 # All rights reserved.
 #
 # This copy of SIP is licensed for use under the terms of the SIP License
@@ -41,7 +41,6 @@ from ..specification import (AccessSpecifier, Argument, ArgumentType,
         Specification, Transfer, TypeHints, WrappedClass, WrappedException,
         WrappedEnum, WrappedEnumMember)
 from ..templates import encoded_template_name, same_template_signature
-from ..type_hints import get_type_hint
 from ..utils import (argument_as_str, cached_name, find_iface_file,
         normalised_scoped_name, same_base_type)
 
@@ -56,7 +55,8 @@ class ParserManager:
     """
 
     def __init__(self, hex_version, encoding, abi_version, tags,
-            disabled_features, protected_is_public, include_dirs, strict):
+            disabled_features, protected_is_public, include_dirs, sip_module,
+            is_strict):
         """ Initialise the manager. """
 
         # Create the lexer.
@@ -78,12 +78,15 @@ class ParserManager:
         self.tags = tags
 
         self.spec = Specification(
-                tuple([int(v) for v in abi_version.split('.')]), strict)
+                tuple([int(v) for v in abi_version.split('.')]), is_strict,
+                sip_module)
+
+        # The module is initially unnamed.
+        self.modules = [self.spec.module]
 
         self.c_bindings = None
         self.code_block = None
         self.module_state = None
-        self.module_states = []
         self.paren_depth = 0
         self.parsing_template = False
         self.parsing_virtual = False
@@ -1201,14 +1204,9 @@ class ParserManager:
         sip_file, raw_sip_file, input, lineno, lexpos, module_state = self._file_stack.pop()
 
         if module_state is None:
+            self._import_module(self._sip_file)
             module_state = self.module_state
 
-            # The current file was %Included so create a new Module and
-            # ModuleState as if it had been %Imported.
-            module = Module()
-            self.spec.modules.append(module)
-            self.module_state = ModuleState(module, self._sip_file, self)
-
         self._file_stack.append(
                 (sip_file, raw_sip_file, input, lineno, lexpos, module_state))
 
@@ -1316,7 +1314,7 @@ class ParserManager:
     def find_qualifier(self, p, symbol, name, required=True):
         """ Return a Qualifier or None if one doesn't exist. """
 
-        for module in self.spec.modules:
+        for module in self.modules:
             for qual in module.qualifiers:
                 if qual.name == name:
                     return qual
@@ -1418,31 +1416,27 @@ class ParserManager:
         None if none were specified.
         """
 
-        th_text = annotations.get('TypeHint')
-        th_in_text = annotations.get('TypeHintIn')
-        th_out_text = annotations.get('TypeHintOut')
+        th = annotations.get('TypeHint')
+        th_in = annotations.get('TypeHintIn')
+        th_out = annotations.get('TypeHintOut')
         th_value = annotations.get('TypeHintValue')
 
-        if th_in_text is None:
-            th_in_text = th_text
-        elif th_text is not None:
+        if th_in is None:
+            th_in = th
+        elif th is not None:
             self.parser_error(p, symbol,
                     "'TypeHint' and 'TypeHintIn' cannot both be specified")
 
             return None
 
-        th_in = None if th_in_text is None else get_type_hint(self.spec, th_in_text)
-
-        if th_out_text is None:
-            th_out_text = th_text
-        elif th_text is not None:
+        if th_out is None:
+            th_out = th
+        elif th is not None:
             self.parser_error(p, symbol,
                     "'TypeHint' and 'TypeHintOut' cannot both be specified")
 
             return None
 
-        th_out = None if th_out_text is None else get_type_hint(self.spec, th_out_text)
-
         if th_in is not None or th_out is not None or th_value is not None:
             # Check that type hints haven't been suppressed.
             if annotations.get('NoTypeHint') is not None:
@@ -1459,9 +1453,7 @@ class ParserManager:
         code will be generated for.
         """
 
-        module = self.module_state.module
-
-        return module is self.spec.modules[0] or module.composite is not None
+        return self.module_state.module is self.spec.module
 
     def instantiate_class_template(self, p, symbol, fq_cpp_name, template,
             py_name, no_type_name, docstring):
@@ -1490,9 +1482,10 @@ class ParserManager:
                         column=self._get_column_from_lexpos(t.lexpos)))
 
     def parse(self, sip_file):
-        """ Parse a .sip file and return a Specification object and a list of
-        the .sip files that specify the module to be generated.  A
-        UserException is raised if there was an error.
+        """ Parse a .sip file and return a 3-tuple of a Specification object, a
+        list of Module objects and a list of the .sip files that specify the
+        module to be generated.  A UserException is raised if there was an
+        error.
         """
 
         # Note that the retention of the 'raw' filename, ie. that which was
@@ -1502,7 +1495,7 @@ class ParserManager:
         raw_sip_file = sip_file
         sip_file = os.path.abspath(sip_file)
 
-        self.module_state = ModuleState(self.spec.modules[0], sip_file, self)
+        self.module_state = ModuleState(self.spec.module, sip_file)
 
         try:
             self._parser.parse(self._read(sip_file, raw_sip_file),
@@ -1541,7 +1534,7 @@ class ParserManager:
         for klass in self._template_arg_classes:
             self.spec.classes.remove(klass)
 
-        return self.spec, self._sip_files
+        return self.spec, self.modules, self._sip_files
 
     def parser_error(self, p, symbol, text):
         """ Record an error caused by a symbol in a production. """
@@ -1636,47 +1629,17 @@ class ParserManager:
                         "'{0}' is being read recursively".format(sip_file))
                 return
 
+        # Ignore the file if we have already read it.
+        if sip_file in self._all_sip_files:
+            return
+
         if new_module:
-            importing_from = self.module_state.module
-
-            # Create a new module if it has not already been defined.
-            for module_state in self.module_states:
-                if module_state.sip_file == sip_file:
-                    module = module_state.module
-                    break
-            else:
-                module = Module()
-                self.spec.modules.append(module)
-
-                module.default_exception = self.module_state.module.default_exception
-                old_module_state = self.module_state
-                self.module_state = ModuleState(module, sip_file, self)
-
-                # Get the configuration of the new module.
-                mod_tags, mod_disabled = get_bindings_configuration(
-                        self.spec.abi_version[0], sip_file, self._include_dirs)
-
-                for tag in mod_tags:
-                    if tag not in self.tags:
-                        self.tags.append(tag)
-
-                for feature in mod_disabled:
-                    if feature not in self._disabled_features:
-                        self._disabled_features.append(feature)
-
-            # Add the new import unless it has already been imported.
-            if module not in importing_from.imports:
-                importing_from.imports.insert(0, module)
+            old_module_state = self.module_state
+            self._import_module(sip_file)
         else:
             # This means that the file was %Included rather than %Imported.
             old_module_state = None
 
-        # Ignore the file if we have already read it. This replicates the
-        # behaviour of the old parser but shouldn't be required for a well
-        # specified project.
-        if sip_file in self._all_sip_files:
-            return
-
         # Save the state of the current .sip file.
         self._file_stack.append(
                 (self._sip_file, self.raw_sip_file, self._input,
@@ -2115,6 +2078,34 @@ class ParserManager:
         # call_super_init defaults to False if it wasn't specified.
         module.call_super_init = bool(module_state.call_super_init)
 
+    def _import_module(self, sip_file):
+        """ Create a new Module object and corresponding ModuleState object for
+        a .sip file and make it current.
+        """
+
+        importing_from = self.module_state.module
+
+        module = Module()
+        self.modules.append(module)
+
+        module.default_exception = self.module_state.module.default_exception
+        self.module_state = ModuleState(module, sip_file)
+
+        # Get the configuration of the new module.
+        mod_tags, mod_disabled = get_bindings_configuration(
+                self.spec.abi_version[0], sip_file, self._include_dirs)
+
+        for tag in mod_tags:
+            if tag not in self.tags:
+                self.tags.append(tag)
+
+        for feature in mod_disabled:
+            if feature not in self._disabled_features:
+                self._disabled_features.append(feature)
+
+        # Add the new import.
+        importing_from.imports.append(module)
+
     def _read(self, sip_file, raw_sip_file):
         """ Return the contents of the current .sip file. """
 
@@ -2148,7 +2139,7 @@ class ParserManager:
 class ModuleState:
     """ Encapsulate the parser-related state for a module. """
 
-    def __init__(self, module, sip_file, pm):
+    def __init__(self, module, sip_file):
         """ Initialise the state. """
 
         self.module = module
@@ -2161,8 +2152,6 @@ class ModuleState:
         self.kw_args = KwArgs.NONE
         self.nr_timelines = 0
 
-        pm.module_states.append(self)
-
 
 class ScopeState:
     """ Encapsulate the parser-related state for a scope. """
